local Rect = require("structures/rect")
local Vector = require("structures/vector")
local Object = require("libraries/classic")
local level = require("objects/level")
local player = require("objects/player")
local Box = Object:extend()

function Box:new (x, y, width, height)
	self.rect = Rect(x, y, width, height)
	self.velocity = Vector(0, 0)
	self.gravityStrength = 250
	self.recordTimer = 0
	self.pastPositions = {}
end

function Box:update (deltaTime)
	self.recordTimer = self.recordTimer + deltaTime
	if self.recordTimer > 0.01 then -- Only record once every 0.01 seconds.
		self.recordTimer = 0
		if player.rewinding and #self.pastPositions > 0 then -- Restore the stored position and velocity.
			self.rect.x = self.pastPositions[#self.pastPositions][1]
			self.rect.y = self.pastPositions[#self.pastPositions][2]
			self.velocity.x = self.pastPositions[#self.pastPositions][3]
			self.velocity.y = self.pastPositions[#self.pastPositions][4]
			if #self.pastPositions > 1 then
				self.pastPositions[#self.pastPositions] = nil
			end
		else
			table.insert(self.pastPositions, {self.rect.x, self.rect.y, self.velocity.x, self.velocity.y}) -- Store position and velocity.
		end
	end

	if not (player.rewinding) then -- Only do gravity and physics if the player isn't rewinding.
		self.velocity.y = self.velocity.y + (self.gravityStrength * deltaTime)
		self:move(self.velocity.x * deltaTime, self.velocity.y * deltaTime)	
	end
end

function Box:draw ()
	love.graphics.setColor(1, 0, 0, 1)
	love.graphics.rectangle("fill", self.rect.x, self.rect.y, self.rect.width, self.rect.height)
	love.graphics.setColor(1, 1, 1, 1)
end

-- The same code as the player move function.
function Box:move (velocityX, velocityY)
	self.rect.x = self.rect.x + velocityX
	local overlaps = self.rect:overlaps(level.collidableRects)
	for _, overlap in pairs(overlaps) do
		self.velocity.x = 0
		if velocityX > 0 then
			self.rect.x = overlap.x - self.rect.width
		elseif velocityX < 0 then
			self.rect.x = overlap.x + overlap.width
		end
	end

	self.rect.y = self.rect.y + velocityY
	overlaps = self.rect:overlaps(level.collidableRects)
	for _, overlap in pairs(overlaps) do
		self.velocity.y = 0
		if velocityY > 0 then
			self.rect.y = overlap.y - self.rect.height
		elseif velocityY < 0 then
			self.rect.y = overlap.y + overlap.height
		end
	end
end

return Box
