local Rect = require("structures/rect")
local Vector = require("structures/vector")
local level = require("objects/level")
local player = {
	acceleration = 250,
	deceleration = 250,
	jumpStrength = 100,
	gravityStrength = 250,
	maxMoveSpeed = 50,
	maxFallSpeed = 250,
	velocity = Vector(0, 0),
	rewinding = false,
	rect = nil -- The rect is generated in levelManager.lua
}

-- The update function just contains a basic physics-based player movement system.
function player:update (deltaTime)
	self.velocity.y = self.velocity.y + (self.gravityStrength * deltaTime)

	local keyLeft = 0
	local keyRight = 0
	if love.keyboard.isDown("a", "left") then keyLeft = 1 end
	if love.keyboard.isDown("d", "right") then keyRight = 1 end
	local inputDirection = keyRight - keyLeft
	if inputDirection ~= 0 then self.facing = inputDirection end
	
	if inputDirection ~= 0 and (self.velocity.x == 0 or math.sign(self.velocity.x) == math.sign(inputDirection)) then
		self.velocity.x = math.min(math.abs(self.velocity.x) + (self.acceleration * deltaTime), self.maxMoveSpeed) * inputDirection
	elseif self.velocity.x ~= 0 and (inputDirection == 0 or math.sign(self.velocity.x) ~= math.sign(inputDirection)) then
		self.velocity.x = math.max(math.abs(self.velocity.x) - (self.deceleration * deltaTime), 0) * math.sign(self.velocity.x)
	end
	self.velocity.y = math.min(self.velocity.y, self.maxFallSpeed)
	if love.keyboard.isDown("space", "w", "up") and self:isGrounded() then
		self.velocity.y = -self.jumpStrength
	end

	self:move(self.velocity.x * deltaTime, self.velocity.y * deltaTime)

	self.rewinding = love.keyboard.isDown("lshift") -- Press Left Shift to rewind
end

function player:draw ()
	love.graphics.rectangle("fill", self.rect.x, self.rect.y, self.rect.width, self.rect.height)
end

-- Create a temporary rect to determine if the area under the player is collidable. If so, it is considered ground.
function player:isGrounded ()
	return #Rect(self.rect.x, self.rect.y + self.rect.height, self.rect.width, 1):overlaps(level.collidableRects) > 0
end

-- Most of this code was taken from a video on making a platformer in Python by DaFluffyPotato. https://www.youtube.com/watch?v=abH2MSBdnWc&t=0s. The code basically moves the player and kicks them back.
function player:move (velocityX, velocityY)
	self.rect.y = self.rect.y + velocityY
	overlaps = self.rect:overlaps(level.collidableRects)
	for _, overlap in pairs(overlaps) do
		self.velocity.y = 0
		if velocityY > 0 then
			self.rect.y = overlap.y - self.rect.height
		elseif velocityY < 0 then
			self.rect.y = overlap.y + overlap.height
		end
	end

	self.rect.x = self.rect.x + velocityX
	local overlaps = self.rect:overlaps(level.collidableRects)
	for _, overlap in pairs(overlaps) do
		self.velocity.x = 0
		if velocityX > 0 then
			self.rect.x = overlap.x - self.rect.width
		elseif velocityX < 0 then
			self.rect.x = overlap.x + overlap.width
		end
	end
end

return player
